/**********************************************************************************/
/*                                                                                */
/*        Copyright (C) 2005, WiNRADiO Communications, All Rights Reserved        */
/*                                                                                */
/**********************************************************************************/

#include "u_freqedit.h"

DEFINE_EVENT_TYPE(__EVT_FREQ_CHANGED)

BEGIN_EVENT_TABLE( TFreqEdit, wxTextCtrl )
        EVT_CHAR(TFreqEdit::OnChar)
        EVT_SET_FOCUS(TFreqEdit::OnSetFocus)
        EVT_KILL_FOCUS(TFreqEdit::OnKillFocus)
END_EVENT_TABLE()



TFreqEdit::TFreqEdit(wxWindow *Parent,wxWindowID id):wxTextCtrl(Parent,id,"",wxDefaultPosition,wxDefaultSize,wxTE_RIGHT)
{
    m_Focused=false;
}


void TFreqEdit::SetMinMax(unsigned int Min,unsigned int Max)
{   
    m_Min=Min;
    m_Max=Max;
}

void TFreqEdit::OnChar(wxKeyEvent& event)
{
 int KeyCode;
 
    KeyCode=event.GetKeyCode();
    
    if(!m_Focused)
    {
        return;
    }
    
    if(KeyCode==27) //ESC
    {
        GetParent()->SetFocus();
    }
    else if(KeyCode==13) //enter
    {
        OnTextEnter();
    }
    else
    {
        event.Skip();
    }
}


void TFreqEdit::OnSetFocus(wxFocusEvent &event)
{
    m_Focused=true;
    event.Skip();
}
    
void TFreqEdit::OnKillFocus(wxFocusEvent &event)
{
    Clear();
    WriteText(m_Buffer);
    event.Skip();    
    m_Focused=false;
}

bool TFreqEdit::StrFreqToL(const char *freq,unsigned long long *Value,TFreqUnits *Units)
{
 char *Buff,*Ptr;
 double Result;
 int Dots;


    if(freq)
    {
        while(*freq && *freq<=0x20)
        {
            freq++;
        }

        if(*freq)
        {
            Buff=(char*)malloc(strlen(freq)+1);
            Ptr=Buff;

            *Ptr=0;

            Dots=0;

            while(*freq && (*freq<='9' || *freq=='.'))
            {
                if(*freq<'0' && *freq!='.')
                {
                    if(*freq==' ' || *freq=='\t')
                    {
                        break;
                    }
                    else
                    {
                        free(Buff);
                        return false;
                    }
                }
                
                if(*freq=='.')
                {
                    if(Dots)
                    {
                        return false;
                    }
                    Dots++;                    
                }

                *Ptr++=*freq++;
            }

            

            if(*Buff==0)
            {                
                free(Buff);
                return false;
            }
            else
            {
                *Ptr=0;

                 Result=atof(Buff);


                 while(*freq && *freq<=0x20)
                 {
                     freq++;
                 }

                 Ptr=Buff;

                 *Ptr=0;

                 while(*freq && *freq<=0x20)
                 {
                    freq++;
                 }

                 while(*freq && *freq>0x20)
                 {
                     *Ptr++=*freq++;
                 }

                *Units=units_Hz;

                 if(*Buff)
                 {
                    *Ptr=0;
		    if(strcasecmp(Buff,"")==0)
		    {
			Result=Result*1000;
			*Units=units_kHz;
		    }
                    else if(strcasecmp(Buff,"Hz")==0)
                    {
                    }
                    else if(strcasecmp(Buff,"kHz")==0 || strcasecmp(Buff,"k")==0)
                    {
                        Result=Result*1000;
                        *Units=units_kHz;
                    }
                    else if(strcasecmp(Buff,"MHz")==0 || strcasecmp(Buff,"M")==0)
                    {
                        Result=Result*1000000;
                        *Units=units_MHz;
                    }
                    else if(strcasecmp(Buff,"GHz")==0 || strcasecmp(Buff,"G")==0)
                    {
                        Result=Result*1000000000;
                        *Units=units_GHz;
                    }
                    else
                    {
                        free(Buff);
                        return false;
                    }
                 }
		 else
		 {
		    Result=Result*1000;
		    *Units=units_kHz;
		 }

                 free(Buff);

                 *Value=(unsigned long long)Result;

                 return true;
            }
        }
        else
        {
            return false;
        }
    }
    else
    {
        return false;
    }

    return false;
}

bool TFreqEdit::OnTextEnter(void)
{
 unsigned long long LongFreq;
 unsigned int Freq;
 TFreqUnits Units;
 wxString Text; 
 wxString Buffer,Save;
 
    Save=m_Buffer;
 
    Text=GetLineText(0);
    if(StrFreqToL(Text.c_str(),&LongFreq,&Units))
    {

        Freq=LongFreq;
        
        if(Freq<m_Min || Freq>m_Max)
        {
            m_Buffer=GetLineText(0);
            Buffer.Printf("Value is out of range (%uHz - %uHz).",m_Min,m_Max);
            wxMessageBox(Buffer,"Information",wxOK|wxICON_INFORMATION,GetParent());
            SetFocus();
            m_Buffer=Save;
            return false;
        }
        
        unsigned int OldFreq=m_Freq;
        TFreqUnits OldUnits=m_Units;
        
        SetFreq(Freq,Units);
        
        GetParent()->SetFocus();
        
        wxCommandEvent Event(__EVT_FREQ_CHANGED);
        
        Event.SetId(GetId());
        
        GetParent()->GetEventHandler()->ProcessEvent(Event);
        
        if(Event.GetSkipped())
        {            
            SetFreq(OldFreq,OldUnits);
        }
    }
    else
    {
        m_Buffer=GetLineText(0);
        wxMessageBox("Invalid format of frequency.","Error",wxICON_STOP|wxOK,GetParent());
        SetFocus();
        m_Buffer=Save;
        return false;
    }
    
    return true;
}

void TFreqEdit::SetFreq(unsigned int Freq,TFreqUnits Units)
{
 unsigned int Frac;
 unsigned int Divs[4]={1,1000,1000000,1000000000};
 wxString StrUnits[4]={"Hz","kHz","MHz","GHz"};
 unsigned int DecNums[4]={0,3,6,9};
 unsigned int Div;
 int DecNum;
 wxString Buffer;
 
    //if((int)Units>3)
    //{
        //Units=units_GHz;
    //}
 
    Div=Divs[Units];
    
    Frac=Freq % Div;
    
    m_Freq=Freq;
    m_Units=Units;
        
    if(Frac)
    {
        DecNum=DecNums[Units];
        while(!(Frac%10) && Frac)
        {
            Frac/=10;
            DecNum--;
        }

    }
    else
    {
        DecNum=0;
    }

    if(Frac)
    {
        Buffer.Printf("%u.%0*u %s",Freq/Div,DecNum,Frac,StrUnits[Units].c_str());
    }
    else
    {
        Buffer.Printf("%u %s",Freq/Div,StrUnits[Units].c_str());
    }
    
    m_Buffer=Buffer;
    Clear();
    WriteText(Buffer);
    
    SetInsertionPoint(0);
}

